@@ -47,6 +47,9 @@
 #define CDRECORD_WRITE_CDR "Does write CD-R media"
 #define CDRECORD_NOT_WRITE_CDRW "Does not write CD-RW media"
 #define CDRECORD_WRITE_CDRW "Does write CD-RW media"
+#define CDRECORD_READ_DVD "Does read DVD-ROM media"
+#define CDRECORD_WRITE_DVD "Does write DVD-R media"
+#define CDRECORD_NOT_WRITE_DVD "Does not write DVD-R media"
 #define CDRECORD_NOT_WRITE_DUMMY "Does not support test writing"
 #define CDRECORD_WRITE_DUMMY "Does support test writing"
 #define CDRECORD_MAX_CDREAD_SPEED "Maximum read  speed:"
@@ -58,17 +61,35 @@
 #define CDRECORD_INCOMPATIBLE_MEDIUM "cannot format medium - incompatible medium"
 #define CDRECORD_FIXATING_TIME "Fixating time:"
 
-typedef struct {
-  char *cmdline;
-  char *bus;
-} Tsearch_drive;
 
 /* definition de tous les type de bus */
-Tsearch_drive listesearchdrives[] = {
-    { "", " " }, /* scsi (sg) */
-    { "dev=ATA", "ATA:" }, /* ide sg */
-    { "dev=ATAPI", "ATAPI:" }, /* ide */
-    { NULL, NULL}
+Tsearchdrive listesearchdrives[] = {
+    { "", "", " ", 100 }, /* scsi (sg) */
+    { "dev=ATA:", "ATA:", "ATA", 80 }, /* ide sg */
+    { "dev=ATAPI:", "ATAPI:", "ATAPI", 60 }, /* ide */
+#ifdef LINUX_IDE  /* pure ide devices with linux */
+    { "dev=/dev/hda", "/dev/hda", "IDE", 40 },
+    { "dev=/dev/hdb", "/dev/hdb", "IDE", 40 },
+    { "dev=/dev/hdc", "/dev/hdc", "IDE", 40},
+    { "dev=/dev/hdd", "/dev/hdd", "IDE", 40 },
+    { "dev=/dev/hde", "/dev/hde", "IDE", 40 },
+    { "dev=/dev/hdf", "/dev/hdf", "IDE", 40 },
+    { "dev=/dev/hdg", "/dev/hdg", "IDE", 40 },
+    { "dev=/dev/hdh", "/dev/hdh", "IDE", 40 },
+    { "dev=/dev/hdi", "/dev/hdi", "IDE", 40 },
+    { "dev=/dev/hdj", "/dev/hdj", "IDE", 40 },
+    { "dev=/dev/hdk", "/dev/hdk", "IDE", 40 },
+    { "dev=/dev/hdl", "/dev/hdl", "IDE", 40 },
+    { "dev=/dev/hdm", "/dev/hdm", "IDE", 40 },
+    { "dev=/dev/hdn", "/dev/hdn", "IDE", 40 },
+    { "dev=/dev/hdo", "/dev/hdo", "IDE", 40 },
+    { "dev=/dev/hdp", "/dev/hdp", "IDE", 40 },
+    { "dev=/dev/hdq", "/dev/hdq", "IDE", 40 },
+    { "dev=/dev/hdr", "/dev/hdr", "IDE", 40 },
+    { "dev=/dev/hds", "/dev/hds", "IDE", 40 },
+    { "dev=/dev/hdt", "/dev/hdt", "IDE", 40 },
+#endif
+    { NULL, NULL, NULL, 0}
   };
 
 gboolean cdrecord_grave_callback(GIOChannel *Astd, GIOCondition Acond, gpointer Adata);
@@ -130,13 +151,15 @@ gboolean extractlecteur(gchar *Achaine, gchar **Adev, gchar **Amarque, gchar **A
 gboolean scan_callback(GIOChannel *Astd, GIOCondition Acond, gpointer Adata)
 {
   GIOStatus Lstatus;
+  GHashTable *Lhash = (GHashTable *)Adata;
+  gint *Lcont = (gint *)g_hash_table_lookup(Lhash, "cont");
+  Tsearchdrive *Ldrivedesc = (Tsearchdrive *)g_hash_table_lookup(Lhash, "drivedesc");
   gchar *Lbuffer = NULL;
-  gchar *Lbustype = (gchar *)Adata;
   gchar *Lmodel, *Lmarque, *Ldev, *Lquoi;
   
   /* fin du callback lorsque l'on recoi un signal comme quoi le pipe est ferme */
   if (Acond == G_IO_HUP || Acond == G_IO_ERR) {
-    *Lbustype = 0;
+    *Lcont = 2;
     return FALSE;
   }
   
@@ -144,15 +167,19 @@ gboolean scan_callback(GIOChannel *Astd, GIOCondition Acond, gpointer Adata)
   if (!Lbuffer) return FALSE;
     
   g_strstrip(Lbuffer);  
- _DEB("scan lecteur = %s\n", Lbuffer); 
-  if (extractlecteur(Lbuffer, &Ldev, &Lmarque, &Lmodel, &Lquoi) == TRUE && !is_duplicate_drives(Lbustype, Ldev)) {
- _DEB("oui un lecteur");
+/* _DEB("scan lecteur = %s\n", Lbuffer); */
+  if (extractlecteur(Lbuffer, &Ldev, &Lmarque, &Lmodel, &Lquoi) == TRUE) {
+
+    insert_or_update_drive(Ldev, Lmarque, Lmodel, Lquoi, Ldrivedesc);
+
+    _DEB("oui un lecteur");
+/*    
     g_snprintf(Gldrives[Gnumdrives].dev, sizeof(Gldrives[Gnumdrives].dev)-1, "%s%s", Lbustype, Ldev);
     g_snprintf(Gldrives[Gnumdrives].name, sizeof(Gldrives[Gnumdrives].name)-1,
       "%s %s %s", Lmarque, Lmodel, Lquoi);
     Gldrives[Gnumdrives].type = -1;
     Gldrives[Gnumdrives].vitesse = 0;
-    Gnumdrives++;
+    Gnumdrives++;*/
     
     g_free(Lquoi);
     g_free(Lmodel);
@@ -166,28 +193,30 @@ gboolean scan_callback(GIOChannel *Astd, GIOCondition Acond, gpointer Adata)
 }
 
 /* scan des lecteurs */
-gboolean scan_for_drives(GError **Aerror)
+gboolean scan_for_drives(GHashTable *Ahash, GError **Aerror)
 {
   gchar **Lcmd;
   gchar *Lcommandline;
   gint pid, g_out, Lnbrarg;
   gboolean Lbolstatus;
   GIOChannel *Lcom;
-  GIOStatus Lstatus;
+  gboolean Lstatus;
   guint Lcomevent;
-  gchar *Lbustype;
-  Tsearch_drive *Lcurentry;
+  Tsearchdrive *Lcurentry;
+  gint *Lcont = (gint *)g_hash_table_lookup(Ahash, "cont");
+  gboolean *Labort = (gboolean *)g_hash_table_lookup(Ahash, "gabort");
   
-  Gnumdrives=0;
+  free_drives(FALSE);
 
-  for (Lcurentry = listesearchdrives; Lcurentry->cmdline; Lcurentry++) {
-    Lcommandline = g_strdup_printf("%s -scanbus %s", conf_get_string("cdrecord"), Lcurentry->cmdline);
+  for (Lcurentry = listesearchdrives; Lcurentry->detectline && *Labort == FALSE; Lcurentry++) {
+    Lcommandline = g_strdup_printf("%s -scanbus %s", conf_get_string("cdrecord"), Lcurentry->detectline);
   _DEB("commande =%s\n", Lcommandline);
     Lstatus = g_shell_parse_argv(Lcommandline, &Lnbrarg, &Lcmd, Aerror);
     g_free(Lcommandline);
-    if (Lstatus == FALSE) {
-      return FALSE;
-    }
+    if (Lstatus == FALSE) return FALSE;
+
+    *Lcont = 3;
+    g_hash_table_insert(Ahash, "drivedesc", Lcurentry);
 
     Lbolstatus = g_spawn_async_with_pipes(NULL, Lcmd, NULL, /* env argument */
       (GSpawnFlags ) (0),
@@ -199,31 +228,28 @@ gboolean scan_for_drives(GError **Aerror)
       return FALSE;
     }
 
-    Lbustype = g_strdup(Lcurentry->bus);
-
     Lcom = g_io_channel_unix_new( g_out );
     g_io_channel_set_encoding (Lcom, NULL, NULL);
     g_io_channel_set_flags( Lcom, G_IO_FLAG_NONBLOCK, NULL );
     Lcomevent = g_io_add_watch (Lcom, (G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_PRI),
   					     scan_callback,
-					     Lbustype);
-  
-    while (*Lbustype) {
+					     Ahash); 
+    while (*Lcont==3) {
       gtk_main_iteration();    
     }
-    
+
     g_source_remove(Lcomevent);
       
-    g_free(Lbustype);
     g_io_channel_shutdown(Lcom, FALSE, NULL);
     g_io_channel_unref(Lcom);
     g_spawn_close_pid(pid);
+    g_hash_table_remove(Ahash, "drivedesc");
   }
 
   return TRUE;
 }
 
-gboolean maj_drive_info(gint Anumdrive, GError **Aerror)
+gboolean maj_drive_info(Tdriveinfo *Adrive, gchar *Adev, GError **Aerror)
 {
   gchar *Lcommandline, *Lout = NULL, *Lerr = NULL;
   gchar *s = NULL, *f;
@@ -231,7 +257,8 @@ gboolean maj_drive_info(gint Anumdrive, GError **Aerror)
   gboolean Lstatus;
   gint Ltype = 0;
 
-  Lcommandline = g_strdup_printf("%s -prcap dev=%s", conf_get_string("cdrecord"), Gldrives[Anumdrive].dev);
+  Lcommandline = g_strdup_printf("%s -prcap dev=%s", conf_get_string("cdrecord"), Adev);
+_DEB("==================%s\n", Lcommandline);  
   Lstatus = g_spawn_command_line_sync(Lcommandline, &Lout, &Lerr, &Lexit, Aerror);
   g_free(Lcommandline);
   if (Lstatus == FALSE || !Lout || !Lerr || !*Lout) {
@@ -244,6 +271,8 @@ gboolean maj_drive_info(gint Anumdrive, GError **Aerror)
   if (strstr(Lout, CDRECORD_WRITE_CDR)) Ltype += _WRITE_CDR;
   if (strstr(Lout, CDRECORD_WRITE_CDRW)) Ltype += _WRITE_CDRW;
   if (strstr(Lout, CDRECORD_WRITE_DUMMY)) Ltype += _WRITE_DUMMY;
+  if (strstr(Lout, CDRECORD_READ_DVD)) Ltype += _READ_DVD;
+  if (strstr(Lout, CDRECORD_WRITE_DVD)) Ltype += _WRITE_DVD;
 
   /* vitesse de lecture maximum */
   if ((s=strstr(Lout, CDRECORD_MAX_CDREAD_SPEED))) {
@@ -261,10 +290,10 @@ gboolean maj_drive_info(gint Anumdrive, GError **Aerror)
 
   if (!s) return FALSE;
 
-  Gldrives[Anumdrive].type = Ltype; /* type: lecteur ou graveur */
-  Gldrives[Anumdrive].vitesse = atoi(s);  /* vitesse de lecture ou de gravure maximum */
-
-  _DEB("donc la vitesse de lecture => %d\n", Gldrives[Anumdrive].vitesse  );
+  Adrive->type = Ltype; /* type: lecteur ou graveur */
+  Adrive->vitesse = atoi(s);  /* vitesse de lecture ou de gravure maximum */
+  _DEB("== le type => %d\n", Ltype);
+  _DEB("== donc la vitesse de lecture => %d\n", Adrive->vitesse  );
   return TRUE;
 }
 
@@ -288,7 +317,7 @@ gboolean burn_from_image(gchar *Aop, GHashTable *Ahash, GError **Aerror) {
   gchar *Lcommandline, *Lbufgrav, *Ltxt;
   GIOChannel *Lcom, *Lcomerr;
   guint Lcomevent, Lcomerrevent;
-  gboolean Lcont = TRUE;
+  gint *Lcont = (gint *) g_hash_table_lookup(Ahash, "cont");
   GtkWindow *Lwindow = GTK_WINDOW(g_hash_table_lookup(Ahash, "window_burn"));
   gboolean Lsimul = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sc_hash_table_lookupv(Ahash, "%ssimul", Aop)));
   gint Lnbrgravure = gtk_spin_button_get_value(GTK_SPIN_BUTTON(sc_hash_table_lookupv(Ahash, "nbrcd%s", Aop)));  
@@ -302,6 +331,7 @@ gboolean burn_from_image(gchar *Aop, GHashTable *Ahash, GError **Aerror) {
   GtkLabel *Ltitle = GTK_LABEL(g_hash_table_lookup(Ahash, "gravetitle"));
   GtkToggleButton *Lbtnnotfix = GTK_TOGGLE_BUTTON(sc_hash_table_lookupv(Ahash, "%snotfix", Aop));
   gboolean Lnotfix = Lbtnnotfix ? gtk_toggle_button_get_active(Lbtnnotfix) : FALSE;
+  gchar **Lextrapara = g_hash_table_lookup(Ahash, "extrapara");
   gboolean Lmulti = FALSE;
   gint *Ldone = (gint *)g_hash_table_lookup(Ahash, "done"); /* fais */
   gint Lcurcd;
@@ -355,13 +385,14 @@ gboolean burn_from_image(gchar *Aop, GHashTable *Ahash, GError **Aerror) {
 
     Lbufvitesse = get_combo_value(Lvitesse);
     Lbufgrav = get_combo_value(Lgraveur);
-    Lcommandline = g_strdup_printf("%s dev=%s -v gracetime=2 %s%s %s %s %s %s %s '%s'",
+    Lcommandline = g_strdup_printf("%s dev=%s -v gracetime=2 %s%s %s %s %s %s %s %s '%s'",
         conf_get_string("cdrecord"), Lbufgrav, *Lbufvitesse != '0' ? "speed=" : "", 
         *Lbufvitesse != '0' ? Lbufvitesse : "",  
         Ldosimul ? " -dummy" : "",  /* simulation ? */
         Ldosimul || Lnotfix ? " -nofix" : "", /* fixer le cd apres ecriture ? */
         Lmulti ? " -multi" : "",  /* multi-session */
         Leject ? "-eject" : "", /* ejecter le cd apres l'operation */
+        Lextrapara && *Lextrapara ? *Lextrapara : "",  /* parametre supplementaire tel que le mode d'ecriture */
         conf_get_string("cdrecordpara"),
         *Liso);
   _DEB("execution [%s]\n", Lcommandline);
@@ -385,8 +416,7 @@ gboolean burn_from_image(gchar *Aop, GHashTable *Ahash, GError **Aerror) {
       return FALSE;
     }
 
-    Lcont = TRUE;
-    g_hash_table_insert(Ahash, "cont", &Lcont);
+    *Lcont = 1;
     g_hash_table_insert(Ahash, "lasttrack", &Llasttrack);
     Lcom = g_io_channel_unix_new( g_out );
     g_io_channel_set_encoding (Lcom, NULL, NULL);
@@ -402,7 +432,7 @@ gboolean burn_from_image(gchar *Aop, GHashTable *Ahash, GError **Aerror) {
     Lcomerrevent = g_io_add_watch (Lcomerr, (G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_PRI),
                                       cdrecord_grave_callback, Ahash); 
     
-    while (Lcont) {
+    while (*Lcont > 0) {
       gtk_main_iteration();    
     }
     g_source_remove(Lcomevent);
@@ -416,7 +446,6 @@ gboolean burn_from_image(gchar *Aop, GHashTable *Ahash, GError **Aerror) {
     *Lpid = 0;
 
     g_hash_table_remove(Ahash, "lasttrack");
-    g_hash_table_remove(Ahash, "cont");
 
     if (*Aerror) {
       _DEB("IL Y A UNE ERREUR !!");
@@ -477,7 +506,7 @@ gboolean cdrecord_grave_callback(GIOChannel *Astd, GIOCondition Acond, gpointer
   GIOStatus Lstatus;
   GHashTable *Lhash = (GHashTable *)Adata;
   gchar Lbuffer[_BUF_SIZE];
-  gboolean *Lcont = (gboolean *) g_hash_table_lookup(Lhash, "cont"); /* on traite encore des donnees ? */
+  gint *Lcont = (gint *) g_hash_table_lookup(Lhash, "cont"); /* on traite encore des donnees ? */
   GError **Lerreur = (GError **) g_hash_table_lookup(Lhash, "gerror"); /* pointeur erreur */
   gint *Ltodo = (gint *) g_hash_table_lookup(Lhash, "todo"); /* nombre de piste a traiter */
   gint *Ldone = (gint *) g_hash_table_lookup(Lhash, "done"); /* nombre de piste deja traite */
@@ -497,7 +526,7 @@ gboolean cdrecord_grave_callback(GIOChannel *Astd, GIOCondition Acond, gpointer
   
   /* fin du callback lorsque l'on recoi un signal comme quoi le pipe est ferme */
   if (Acond == G_IO_HUP || Acond == G_IO_ERR) {
-    (*Lcont) = FALSE;
+    (*Lcont) = 0;
     return FALSE;
   }
   
@@ -612,29 +641,29 @@ _DEB("===>%s", Lbuffer);
   } else if (strstr(Lbuffer, CDRECORD_INPUTOUTPUT_ERROR)) {
     /* erreur entre sortie */
     g_set_error(Lerreur, GRAVEMAN_ERROR, _ERR_NO_CD, _("Error writing CD !"));
-    (*Lcont) = FALSE;
+    (*Lcont) = 0;
     return FALSE;
   } else if (strstr(Lbuffer, CDRECORD_ERRORDISK)) {
     /* erreur pas de cd vierge */
     g_set_error(Lerreur, GRAVEMAN_ERROR, _ERR_NO_CD, _("Error, a CD-R/CD-RW is required in the cd recorder !"));
-    (*Lcont) = FALSE;
+    (*Lcont) = 0;
     return FALSE;
   } else if (strstr(Lbuffer, CDRECORD_BAD_RECORDER)) {
     /* erreur pas de cd vierge */
     g_set_error(Lerreur, GRAVEMAN_ERROR, _ERR_BAD_RECORDER, _("CD recorder unsupported !"));
-    (*Lcont) = FALSE;
+    (*Lcont) = 0;
     return FALSE;
   } else if ((strstr(Lbuffer, CDRECORD_INAPPROPRIATE))) {
     /* erreur fichier audio non compatible */
     g_snprintf(Lsbuf, sizeof(Lsbuf)-1, _("Error with track %.0f: inappropriate audio coding !"), Ltotaldone);
     g_set_error(Lerreur, GRAVEMAN_ERROR, _ERR_INAPPROPRIATE_DATA, Lsbuf);
-    (*Lcont) = FALSE;
+    (*Lcont) = 0;
     return FALSE;
   } else if ((f=strstr(Lbuffer, CDRECORD_NOSUCHFILE))) { 
     /* erreur fichier image iso source introuvable ! */
     g_snprintf(Lsbuf, sizeof(Lsbuf)-1, _("Cannot open iso image !"));
     g_set_error(Lerreur, GRAVEMAN_ERROR, _ERR_INAPPROPRIATE_DATA, Lsbuf);
-    (*Lcont) = FALSE;
+    (*Lcont) = 0;
     return FALSE;
   } else if ((f=strstr(Lbuffer, conf_get_string("cdrecord")))) {
     f=f+strlen(conf_get_string("cdrecord"))+1;
@@ -643,7 +672,7 @@ _DEB("===>%s", Lbuffer);
       Ltxt = _UTF8(f);
       g_set_error(Lerreur, GRAVEMAN_ERROR, _ERR_INAPPROPRIATE_DATA, Ltxt);
       g_free(Ltxt);
-      (*Lcont) = FALSE;
+      (*Lcont) = 0;
       return FALSE;
     }
   } 
@@ -663,7 +692,7 @@ gboolean gravepiste(GHashTable *Ahash, GError **Aerror) {
   gchar *Lcommandline;
   GIOChannel *Lcom, *Lcomerr;
   guint Lcomevent, Lcomerrevent;
-  gboolean Lcont;
+  gint *Lcont = (gint *) g_hash_table_lookup(Ahash, "cont"); /* on traite encore des donnees ? */
   /* nombre d'element total */
   gint *Ldone = (gint *)g_hash_table_lookup(Ahash, "done"); /* fais */
   gint *Ltodo = (gint *) g_hash_table_lookup(Ahash, "todo");
@@ -728,8 +757,7 @@ _DEB("execution [%s]\n", Lcommandline);
       return FALSE;
     }
 
-    Lcont = TRUE;
-    g_hash_table_insert(Ahash, "cont", &Lcont);
+    *Lcont = 1;
     g_hash_table_insert(Ahash, "lasttrack", &Llasttrack);
     Lcom = g_io_channel_unix_new( g_out );
     g_io_channel_set_encoding (Lcom, NULL, NULL);
@@ -745,7 +773,7 @@ _DEB("execution [%s]\n", Lcommandline);
     Lcomerrevent = g_io_add_watch (Lcomerr, (G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_PRI),
                                         cdrecord_grave_callback, Ahash);  
     
-    while (Lcont) {
+    while (*Lcont>0) {
       gtk_main_iteration(); 
     }
 
@@ -760,7 +788,6 @@ _DEB("execution [%s]\n", Lcommandline);
     *Lpid = 0;
 
     g_hash_table_remove(Ahash, "lasttrack");
-    g_hash_table_remove(Ahash, "cont");
 
     if (*Aerror) {
       Lstatus = FALSE;
@@ -838,6 +865,11 @@ _DEB("===>%s", Lbuffer);
     GError **Lerreur = (GError **) g_hash_table_lookup(Lhash, "gerror"); /* pointeur erreur */
     if (!*Lerreur)
       g_set_error(Lerreur, GRAVEMAN_ERROR, _ERR_INCOMPATIBLE_MEDIUM, _("Cannot blank disk, this is not a CD-RW !"));
+  } else if (strstr(Lbuffer, CDRECORD_ERRORDISK)) {
+    /* erreur pas de cd */
+    GError **Lerreur = (GError **) g_hash_table_lookup(Lhash, "gerror"); /* pointeur erreur */
+    if (!*Lerreur)
+      g_set_error(Lerreur, GRAVEMAN_ERROR, _ERR_NO_CD, _("Error, a CD-R/CD-RW is required in the cd recorder !"));
   }
 
   g_free(Lbuffer);
